; ---------------------------------------------------------------------------
; Fuentes del artículo "Añade música a tus programas de PCW"
; Solo música
; (c) SyX 2012
; Sientete libre de usar y abusar de este código.
; ---------------------------------------------------------------------------

; Ensamblar con: pasmo musica.s musica.com

; ---------------------------------------------------------------------------
; Macros para simplificar el código
; ---------------------------------------------------------------------------
; Guardamos en la pila todos los registros
 MACRO PUSH_ALL
    PUSH AF
    PUSH BC
    PUSH DE
    PUSH HL
    EX   AF,AF'
    EXX
    PUSH AF
    PUSH BC
    PUSH DE
    PUSH HL
    PUSH IX
    PUSH IY
 ENDM

; Restauramos todos los registros desde la pila
 MACRO POP_ALL
    POP  IY
    POP  IX
    POP  HL
    POP  DE
    POP  BC
    POP  AF
    EXX
    EX   AF,AF'
    POP  HL
    POP  DE
    POP  BC
    POP  AF
 ENDM

; ---------------------------------------------------------------------------
    ORG  $0100

; ---------------------------------------------------------------------------
; Comienzo del programa
inicio
    ; Mostramos los créditos
    LD   HL,mensaje
    CALL muestra_texto

    ; Tomamos el control de la máquina
    DI
    IM   1
    LD   A,$C3
    LD   HL,int_handler     ; Establecemos nuestro manejador de interrupciones
    LD   ($0038),A
    LD   ($0039),HL

    ; Copiamos la canción a su destino (la compilamos paraq $4000)
    LD   HL,cancion
    LD   DE,$4000
    LD   BC,fin_cancion - cancion
    LDIR

    LD   SP,fin_cancion     ; Inicializamos la pila
    EI

    ; Inicializamos la canción
    LD   DE,cancion         ; Canción que queremos reproducir
    CALL PLY_Init

.bucle_infinito
    ; Nos sincronizamos con el refresco de pantalla
.espera_refresco
    IN   A,($F8)
    BIT  6,A
    JR   Z,.espera_refresco
    IN   A,($F8)
    BIT  6,A
    JR   Z,.espera_refresco
    
    ; Esperamos a que se pase el refresco antes de llamar al reproductor
.espera_fin_refresco
    IN   A,($F8)
    BIT  6,A
    JR   NZ,.espera_fin_refresco

    PUSH_ALL        ; Guardamos en la pila todos los registros usados por el reproductor
    
    CALL PLY_Play   ; Reproducimos nuestra canción

    POP_ALL         ; Restauramos los registros desde la pila


    JR   .bucle_infinito

    CALL PLY_Stop           ; Aquí nunca llegamos :P

; ---------------------------------------------------------------------------
; Manejador de interrupciones
; ---------------------------------------------------------------------------
int_handler
    PUSH  AF
    IN    A,($F4)           ; Reactivamos las interrupciones
    POP   AF
    EI
    RETI

; ---------------------------------------------------------------------------
; Imprime una cadena terminada en $FF usando el firmware
; Entradas:
;    HL : Puntero a la cadena
; ---------------------------------------------------------------------------
muestra_texto
    LD   A,(HL)
.bucle_muestra_texto
    LD   E,A
    LD   C,2
    PUSH HL
    CALL 5              ; BDOS_C_WRITE
    LD   E,7
    LD   C,2
    CALL 5              ; BDOS_C_WRITE
    POP  HL
    INC  HL
    LD   A,(HL)
    CP   $FF
    JR   NZ,.bucle_muestra_texto
    RET

; ---------------------------------------------------------------------------
; Insertamos el código del reproductor del Arkos Tracker
; ---------------------------------------------------------------------------
    include "ArkosTrackerPlayer_PCW.asm"

; ---------------------------------------------------------------------------
; Insertamos la canción "compilada" para $4000 con el comando:
; AKSToBIN.exe -a 0x4000 cancion.aks cancion.bin
; ---------------------------------------------------------------------------
cancion
    incbin "cancion.bin"
fin_cancion

; ---------------------------------------------------------------------------
mensaje
    DEFB 'You are hearing... TAKE OFF by McKlain',13,10
    DEFB 'Arkos Player v1.01 (PCW code by SyX)'
    DEFB $FF

; ---------------------------------------------------------------------------
    END
